//+------------------------------------------------------------------+
//|                                                            va_sum|
//|                                              Copyright 2013,  J.B|
//+------------------------------------------------------------------+
#property copyright "Copyright  2013, J.B"
#property version   "1.00"

#include <MovingAverages.mqh>

#property indicator_buffers 10
#property indicator_plots   1

#property indicator_separate_window
//#property indicator_chart_window

#property indicator_type1   DRAW_LINE
#property indicator_color1  clrRed
#property indicator_width1  3
#property indicator_label1  "sum"

enum viz
  {EMA,vaMA,d1,d2,d2_dir,sd2,dsd2,sum_dsd2,s};

   input int         vaMA_period=15;
   input bool        use_double_smooth=1;
   input bool        bar_exist=1;
   input int         count=7200;
   input int         spread=2;
   input viz         VIZ_buff=7;
   
   double            viz_buffer[],EMA[],vaMA[],d1[],d2[],d2_dir[],sd2[],dsd2[],sum_dsd2[],s[];

   //+------------------------------------------------------------------+
   //| vaMA initialization function                                     |
   //+------------------------------------------------------------------+  
   void OnInit()
     {
      SetIndexBuffer(0,viz_buffer,INDICATOR_DATA);

      SetIndexBuffer(1,EMA,INDICATOR_CALCULATIONS);
      SetIndexBuffer(2,vaMA,INDICATOR_CALCULATIONS);
      SetIndexBuffer(3,d1,INDICATOR_CALCULATIONS);
      SetIndexBuffer(4,d2,INDICATOR_CALCULATIONS);
      SetIndexBuffer(5,d2_dir,INDICATOR_CALCULATIONS);
      SetIndexBuffer(6,sd2,INDICATOR_CALCULATIONS);
      SetIndexBuffer(7,dsd2,INDICATOR_CALCULATIONS);
      SetIndexBuffer(8,sum_dsd2,INDICATOR_CALCULATIONS);
      SetIndexBuffer(9,s,INDICATOR_CALCULATIONS);
     }
   //+------------------------------------------------------------------+
   //| vaMA iteration function                                          |
   //+------------------------------------------------------------------+
   int OnCalculate(const int rates_total,const int prev_calculated,const int begin,const double &price[])
     {
      PlotIndexSetInteger(0,PLOT_DRAW_BEGIN,rates_total-count);

      //+------------------------------------------------------------------+
      static int lastBar;
      if(rates_total==lastBar && bar_exist) return(rates_total);
      lastBar=rates_total;
      //+------------------------------------------------------------------+

      //+------------------------------------------------------------------+
      int first;
      if(rates_total<vaMA_period)return(0);
      if(prev_calculated==0)first=vaMA_period+begin;
      else first=prev_calculated-1;
      //+------------------------------------------------------------------+

      ExponentialMAOnBuffer(rates_total,prev_calculated,begin,vaMA_period,price,EMA);
      VAShift(EMA,vaMA,first,rates_total,vaMA_period);

      if(use_double_smooth)
         ExponentialMAOnBuffer(rates_total,prev_calculated,begin,vaMA_period/4,vaMA,vaMA);

      double tmp=0;
      for(int bar=first; bar<rates_total; bar++)
        {
         d1[bar]=d2[bar]=d2_dir[bar]=dsd2[bar]=viz_buffer[bar]=sum_dsd2[bar]=s[bar]=0;

         if((vaMA[bar]-vaMA[bar-1])>0) d1[bar]=1;
         else if((vaMA[bar]-vaMA[bar-1])<0) d1[bar]=-1;

         if((d1[bar]-d1[bar-1])>0)      {d2[bar-1]=price[bar-1]; d2_dir[bar-1]=-1;}
         else if((d1[bar]-d1[bar-1])<0) {d2[bar-1]=price[bar-1]; d2_dir[bar-1]=+1;}

         tmp=price[bar];
        }

      for(int bar=first; bar<rates_total; bar++)
        {
         if(d2[bar]) tmp=+d2[bar];
         sd2[bar]=tmp;

         dsd2[bar]=(sd2[bar]-sd2[bar-1])*d1[bar]*10000;

         double d=1;
         if(fabs(dsd2[bar]))
           {
            d=dsd2[bar]/fabs(dsd2[bar]);
            dsd2[bar]=dsd2[bar];
           }
        }

      double sss=0;
      double ss=0;
      for(int bar=rates_total-count; bar<rates_total; bar++)
        {
         if(fabs(dsd2[bar])){
         sss+=dsd2[bar]-spread;
         ss-=spread;}
         
         sum_dsd2[bar]=sss;
         s[bar]=ss;
        }

      for(int bar=first; bar<rates_total; bar++)
        {
         switch(VIZ_buff)
           {
            case 0: viz_buffer[bar]=EMA[bar];       break;
            case 1: viz_buffer[bar]=vaMA[bar];      break;
            case 2: viz_buffer[bar]=d1[bar];        break;
            case 3: viz_buffer[bar]=d2[bar];        break;
            case 4: viz_buffer[bar]=d2_dir[bar];    break;
            case 5: viz_buffer[bar]=sd2[bar];       break;
            case 6: viz_buffer[bar]=dsd2[bar];      break;
            case 7: viz_buffer[bar]=sum_dsd2[bar];  break;
            case 8: viz_buffer[bar]=s[bar];         break;
            default: viz_buffer[bar]=0;
           }
        }
        
      return(rates_total);
     }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+

   //+------------------------------------------------------------------+
   //| Calculate VAShift function                                       |
   //+------------------------------------------------------------------+
   int VAShift(const double &array[],double &outBuffer[],int begin,int rates_total,int period)
     {
      int period_div=period/4;
      double vel,acc,aaa;
      for(int bar=begin; bar<rates_total; bar++)
        {
         vel = array[bar]-array[bar-period_div];
         acc = array[bar]-2*array[bar-period_div]+(array[bar-period_div*2]+array[bar-period_div/2])/2;
         aaa = array[bar]-3*array[bar-period_div]+3*(array[bar-period_div*2]+array[bar-period_div/2])/2-(array[bar-period_div*3]+array[bar-period_div/3])/2;

         outBuffer[bar]=array[bar]+vel+acc+aaa/3;
        }
      return(rates_total);
     }


